
import sys
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from createnegativekeywords import _add_keywords_to_campaign

global last_criterion_id
last_criterion_id = -1


def _page2(
    client,
    customer_id,
    merchant_center_account_id,
    main_campaign_name,
    df_adgroup,
    keywords_allcampaigns
    ):

    if _does_campaign_exist(client, customer_id, main_campaign_name) != True:
    # Queries googleads to see if campaign name is already in use.

        # Creates a budget to be used by the campaign that will be created below.
        budget_resource_name = _add_campaign_budget(client, customer_id, main_campaign_name)

        for i in range(3):
            if i == 0:
                campaign_name = f"{main_campaign_name} Exact"
                keyword_array = keywords_allcampaigns[2]
                _add_bidding_strategy(client, customer_id, campaign_name, 150000)
               
            if i == 1:
                campaign_name = f"{main_campaign_name} Broad"
                keyword_array = keywords_allcampaigns[1]
                _add_bidding_strategy(client, customer_id, campaign_name, 50000)
            if i == 2:
                campaign_name = f"{main_campaign_name} Irrelevant"
                keyword_array = keywords_allcampaigns[0]
                _add_bidding_strategy(client, customer_id, campaign_name, 10000)

            # Create a standard shopping campaign with Youtube, Gmail and Discovery excluded from targeting.
            campaign_resource_name = _add_standard_shopping_campaign(
                client, customer_id, budget_resource_name, merchant_center_account_id, campaign_name, i
            )

            nr_rows = len(df_adgroup.index)

            for index in df_adgroup.index:
                # Create a shopping product ad group. i is the incremental value of the for loop which represents the priority.
                ad_group_resource_name = _add_shopping_product_ad_group(
                    client, customer_id, campaign_resource_name, df_adgroup.iloc[index,0], i
                )
                # Create a shopping product ad group ad.
                _add_shopping_product_ad_group_ad(
                    client, customer_id, ad_group_resource_name
                )
                # Remove potentially added spaces and split into individual array items using comma as separator.
                sku_array = []
                sku_text = str(df_adgroup.iloc[index,1]).strip()
                sku_array = sku_text.split(",")
                # Create product group for all SKUs in sku_array and exclude all others.
                _add_product_group(client, customer_id, ad_group_resource_name, sku_array)

            campaign_id = campaign_resource_name.split("/")
            campaign_id = campaign_id[3]
            #third element in resource name is the campaign id.
            _add_keywords_to_campaign(client, customer_id, campaign_id, keyword_array)
            _add_location_target_to_campaign(client,customer_id,campaign_resource_name)

        else:
            return "There was an error uploading your campaign. Please check your input."
    else:
        #In the case where campaign already exists:
        return "Campaign name already exists."




def _create_negative_target_location(client, campaign_resource_name, location_id):

    #Initialise campaign criterion
    operation = client.get_type("CampaignCriterionOperation")
    criterion = operation.create
    #GeotargetConstantService is used to find the id belonging to different countries.
    geo_target_constant_service = client.get_service("GeoTargetConstantService")
    #Specify which campaign to add the targeted locations.
    criterion.campaign = campaign_resource_name
    criterion.status = client.get_type("CampaignCriterionStatusEnum").CampaignCriterionStatus.ENABLED
    criterion.negative = True
    #Setting the negative location
    criterion.location.geo_target_constant = (
        geo_target_constant_service.geo_target_constant_path(location_id)
    )
    #2840 is usa
    #2156 is china

    return operation


def _create_positive_target_location(client, campaign_resource_name, location_id):

    #Initialise campaign criterion
    operation = client.get_type("CampaignCriterionOperation")
    criterion = operation.create
    #GeotargetConstantService is used to find the id belonging to different countries.
    geo_target_constant_service = client.get_service("GeoTargetConstantService")
    #Specify which campaign to add the targeted locations.
    criterion.campaign = campaign_resource_name
    criterion.status = client.get_type("CampaignCriterionStatusEnum").CampaignCriterionStatus.ENABLED
    criterion.negative = False
    #Setting the positive location
    criterion.location.geo_target_constant = (
        geo_target_constant_service.geo_target_constant_path(location_id)
    )

    #2372 is Ireland
    #2826 is united kingdom

    return operation

def _add_location_target_to_campaign(client, customer_id, campaign_resource_name):

    campaign_criterion_service = client.get_service("CampaignCriterionService")

    operation = client.get_type("CampaignCriterionOperation")
    criterion = operation.create
    criterion.campaign = campaign_resource_name
    criterion.status = client.get_type(
        "CampaignCriterionStatusEnum"
    ).CampaignCriterionStatus.ENABLED

    # operations holds all the calls to be made to the api.
    operations = []
    #2840 is usa
    #2156 is china
    operations.append(_create_negative_target_location(client,campaign_resource_name, "2840"))
    operations.append(_create_negative_target_location(client,campaign_resource_name, "2156"))
    #2372 is Ireland
    #2826 is united kingdom
    operations.append(_create_positive_target_location(client,campaign_resource_name, "2372"))
    operations.append(_create_positive_target_location(client,campaign_resource_name, "2826"))

    criterion_response = (
        campaign_criterion_service.mutate_campaign_criteria(
            customer_id=customer_id,
            operations=operations,
        )
    )
    
    print(f"Location target added: {criterion_response}")



#Below is original code

def next_id():
    #This is needed for creating temporary ids for product groups.
    global last_criterion_id
    last_criterion_id -= 1
    return str(last_criterion_id)

def _add_campaign_budget(client, customer_id, campaign_name):
    """Creates a new campaign budget in the specified client account.
    By default the budget is a shared budget if nothing else is specified. """
    campaign_budget_service = client.get_service("CampaignBudgetService")

    # Create a budget, which can be shared by multiple campaigns.
    campaign_budget_operation = client.get_type("CampaignBudgetOperation")
    campaign_budget = campaign_budget_operation.create
    campaign_budget.name = f"{campaign_name} Budget"
    campaign_budget.delivery_method = client.get_type(
        "BudgetDeliveryMethodEnum"
    ).BudgetDeliveryMethod.STANDARD
    # 30000000 is 30 GBP per day.
    campaign_budget.amount_micros = 30000000

    # Add budget.
    campaign_budget_response = campaign_budget_service.mutate_campaign_budgets(
        customer_id=customer_id, operations=[campaign_budget_operation]
    )

    budget_resource_name = campaign_budget_response.results[0].resource_name

    print(f"Added a budget with resource name: '{budget_resource_name}'")

    return budget_resource_name

def _add_bidding_strategy(client, customer_id, campaign_name, bid_ceiling):
    
    # Create a portfolio bidding strategy.
    bidding_strategy_service = client.get_service("BiddingStrategyService")
    bidding_strategy_operation = client.get_type("BiddingStrategyOperation")
    bidding_strategy = bidding_strategy_operation.create
    bidding_strategy.name = f"{campaign_name} Bidding Strategy"
    target_spend = bidding_strategy.target_spend
    target_spend.cpc_bid_ceiling_micros = bid_ceiling

    # Add portfolio bidding strategy.
    try:
        bidding_strategy_response = bidding_strategy_service.mutate_bidding_strategies(
            customer_id=customer_id, operations=[bidding_strategy_operation]
        )
        bidding_strategy_id = bidding_strategy_response.results[0].resource_name
        print(f'Created portfolio bidding strategy "{bidding_strategy_id}".')
    except GoogleAdsException as exception:
        print(
            f'Request with ID "{exception.request_id}" failed with status '
            f'"{exception.error.code().name}" and includes the following errors:'
        )
        for error in exception.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
    return bidding_strategy_id

def _add_shopping_product_ad_group(client, customer_id, campaign_resource_name, ad_group_name, priority):
    """Creates a new shopping product ad group in the specified campaign."""
    ad_group_service = client.get_service("AdGroupService")

    # Create ad group.
    ad_group_operation = client.get_type("AdGroupOperation")
    ad_group = ad_group_operation.create
    ad_group.name = f"{ad_group_name}"
    ad_group.status = client.enums.AdGroupStatusEnum.ENABLED
    ad_group.campaign = campaign_resource_name
    # Sets the ad group type to SHOPPING_PRODUCT_ADS. This is the only value
    # possible for ad groups that contain shopping product ads.

    if priority == 2:
        #High priority (Irrelevant campaign)
        ad_group.type_ = client.get_type(
            "AdGroupTypeEnum"
        ).AdGroupType.SHOPPING_PRODUCT_ADS
        ad_group.cpc_bid_micros = 10000
        #cpc = 0.01
    elif priority == 1:
        #Medium priority (Broad campaign)
        ad_group.type_ = client.get_type(
            "AdGroupTypeEnum"
        ).AdGroupType.SHOPPING_PRODUCT_ADS
        ad_group.cpc_bid_micros = 50000
        #cpc = 0.05
    elif priority == 0:
        #Low priority (Exact campaign)
        ad_group.type_ = client.get_type(
            "AdGroupTypeEnum"
        ).AdGroupType.SHOPPING_PRODUCT_ADS
        ad_group.cpc_bid_micros = 150000
        #cpc = 0.15
    else:
        print("There was an error processing your request, priority setting does not exist.")

    # Add the ad group.
    ad_group_response = ad_group_service.mutate_ad_groups(
        customer_id=customer_id, operations=[ad_group_operation]
    )

    ad_group_resource_name = ad_group_response.results[0].resource_name

    print(
        "Added a product shopping ad group with resource name "
        f"'{ad_group_resource_name}'."
    )

    return ad_group_resource_name
    # [END add_shopping_product_ad_1]


def _add_standard_shopping_campaign(
    client, customer_id, budget_resource_name, merchant_center_account_id, campaign_name, priority
):
    """Creates a new standard shopping campaign in the specified client account."""
    campaign_service = client.get_service("CampaignService")

    # Create standard shoppping campaign.
    campaign_operation = client.get_type("CampaignOperation")
    campaign = campaign_operation.create
    campaign.name = f"{campaign_name}"

    # Configures settings related to shopping campaigns including advertising
    # channel type and shopping setting.
    campaign.advertising_channel_type = client.get_type(
        "AdvertisingChannelTypeEnum"
    ).AdvertisingChannelType.SHOPPING
    campaign.shopping_setting.merchant_id = merchant_center_account_id

    # Sets the sales country of products to include in the campaign.
    campaign.shopping_setting.sales_country = "GB"
    # Sets the priority of the campaign. Higher numbers take priority over lower
    # numbers. For standard shopping campaigns, allowed values are between 0 and
    # 2, inclusive.
    if priority == 0:
        campaign.shopping_setting.campaign_priority = 0
        campaign.target_spend.cpc_bid_ceiling_micros = 150000
        #cpc = 0.15
    elif priority == 1:
        campaign.shopping_setting.campaign_priority = 1
        campaign.target_spend.cpc_bid_ceiling_micros = 50000
        #cpc = 0.05
    elif priority == 2:
        campaign.shopping_setting.campaign_priority = 2
        campaign.target_spend.cpc_bid_ceiling_micros = 10000
        #cpc = 0.01
    else:
        print("Error: No priority detected")

    # Enables local inventory ads for this campaign.
    campaign.shopping_setting.enable_local = True
    # Disable Youtube, Gmail and Discover from targeting.
    campaign.network_settings.target_content_network = False

    #Set targeting location
    #Exclude
    campaign.geo_target_type_setting.negative_geo_target_type = client.get_type("NegativeGeoTargetTypeEnum").NegativeGeoTargetType.PRESENCE
    #Target
    campaign.geo_target_type_setting.positive_geo_target_type = client.get_type(
        "PositiveGeoTargetTypeEnum").PositiveGeoTargetType.PRESENCE


    # Recommendation: Set the campaign to PAUSED when creating it to prevent the
    # ads from immediately serving. Set to ENABLED once you've added targeting
    # and the ads are ready to serve.
    campaign.status = client.get_type(
        "CampaignStatusEnum"
    ).CampaignStatus.PAUSED

    #Uncomment below if you want to use enhanced cpc.
    #campaign.manual_cpc.enhanced_cpc_enabled = False



    # Sets the budget.
    campaign.campaign_budget = budget_resource_name

    # Add the campaign.
    campaign_response = campaign_service.mutate_campaigns(
        customer_id=customer_id, operations=[campaign_operation]
    )

    campaign_resource_name = campaign_response.results[0].resource_name

    print(
        "Added a standard shopping campaign with resource name "
        f"'{campaign_resource_name}'."
    )

    return campaign_resource_name
    # [END add_shopping_product_ad_2]

def _add_shopping_product_ad_group_ad(
    client, customer_id, ad_group_resource_name
):
    """Creates a new shopping product ad group ad in the specified ad group."""
    ad_group_ad_service = client.get_service("AdGroupAdService")

    # Creates a new ad group ad and sets the product ad to it.
    ad_group_ad_operation = client.get_type("AdGroupAdOperation")
    ad_group_ad = ad_group_ad_operation.create
    ad_group_ad.ad_group = ad_group_resource_name
    ad_group_ad.status = client.get_type(
        "AdGroupAdStatusEnum"
    ).AdGroupAdStatus.ENABLED
    client.copy_from(
        ad_group_ad.ad.shopping_product_ad,
        client.get_type("ShoppingProductAdInfo"),
    )

    # Add the ad group ad.
    ad_group_ad_response = ad_group_ad_service.mutate_ad_group_ads(
        customer_id=customer_id, operations=[ad_group_ad_operation]
    )

    ad_group_ad_resource_name = ad_group_ad_response.results[0].resource_name
    print(
        f"Created shopping product ad group ad '{ad_group_ad_resource_name}'."
    )

    return ad_group_resource_name


def _add_product_group(client, customer_id, ad_group_resource_name, array_sku):

    ad_group_criterion_service = client.get_service("AdGroupCriterionService")
    operations = []

    # Create the group tree "root". The root can never have a bidding or it will error.
    # The root node is often called subdivision node in documentation.

    # Get ad group ID
    googleservice = client.get_service("GoogleAdsService")
    query = f"""
              SELECT ad_group.id, ad_group.cpc_bid_micros
              FROM ad_group 
              WHERE ad_group.resource_name = '{ad_group_resource_name}'

      """

    #get ad_group_id and the cpc that was assigned to the ad group.
    search_request = client.get_type("SearchGoogleAdsStreamRequest")
    search_request.customer_id = customer_id
    search_request.query = query
    response = googleservice.search_stream(search_request)
    for batch in response:
        for row in batch.results:
            ad_group = row.ad_group
            ad_group_id = ad_group.id
            ad_group_cpc = ad_group.cpc_bid_micros




    ad_group_criterion_root_operation = _create_listing_group_subdivision(
        client, customer_id, ad_group_id
    )

    # Create temporary resource name in format "customer/CUSTOMERID/adGroupCriteria/ADGROUPID"
    ad_group_criterion_root_resource_name = (
        ad_group_criterion_root_operation.create.resource_name
    )
    operations.append(ad_group_criterion_root_operation)

    productid_dimension_info = client.get_type("ListingDimensionInfo")

    for item in array_sku:
        productid_dimension_info.product_item_id.value = str(item)
        operations.append(
            _create_listing_group_unit_biddable(
                client,
                customer_id,
                ad_group_id,
                ad_group_criterion_root_resource_name,
                productid_dimension_info,
                ad_group_cpc,
                included_product=True
            )
        )

    # Sub-division node: (Condition "other" node)
    # * Product Condition: (not specified)
    # Note that all sibling nodes must have the same dimension type, even if
    # they don't contain a bid.
    client.copy_from(
        productid_dimension_info.product_item_id, client.get_type("ProductItemIdInfo")
    )
    operations.append(
        _create_listing_group_unit_biddable(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_root_resource_name,
            productid_dimension_info,
            cpc_bid_micros=None,
            included_product=False

        )
    )


    # Add the ad group criteria.
    mutate_ad_group_criteria_response = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=operations
        )
    )

    print(
        "added ad group criteria for the listing group tree with the "
        "following resource names:"
    )
    for result in mutate_ad_group_criteria_response.results:
        print(f"\t{result.resource_name}")

    print(f"{len(mutate_ad_group_criteria_response.results)} criteria added.")


def _create_listing_group_subdivision(
        client,
        customer_id,
        ad_group_id,
        parent_ad_group_criterion_resource_name=None,
        listing_dimension_info=None,

):
    """Creates a new criterion containing a subdivision listing group node.

    If the parent ad group criterion resource name or listing dimension info are
    not specified, this method creates a root node.

    Arguments:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID to which the node will be added.
        parent_ad_group_criterion_resource_name: The string resource name of the
            parent node to which this listing will be attached.
        listing_dimension_info: A ListingDimensionInfo object containing details
            for this listing.

    Returns:
        An AdGroupCriterionOperation containing a populated ad group criterion.
    """
    # Create an ad group criterion operation and populate the criterion.
    operation = client.get_type("AdGroupCriterionOperation")
    ad_group_criterion = operation.create
    # The resource name the criterion will be created with. This will define
    # the ID for the ad group criterion.

    ad_group_criterion.resource_name = client.get_service(
        "AdGroupCriterionService"
    ).ad_group_criterion_path(customer_id, ad_group_id, next_id())


    ad_group_criterion.status = client.get_type(
        "AdGroupCriterionStatusEnum"
    ).AdGroupCriterionStatus.ENABLED


    listing_group_info = ad_group_criterion.listing_group
    # Set the type as a SUBDIVISION, which will allow the node to be the
    # parent of another sub-tree.
    listing_group_info.type_ = client.get_type(
        "ListingGroupTypeEnum"
    ).ListingGroupType.SUBDIVISION
    # If parent_ad_group_criterion_resource_name and listing_dimension_info
    # are not null, create a non-root division by setting its parent and case
    # value.
    if (
            parent_ad_group_criterion_resource_name
            and listing_dimension_info != None
    ):
        # Set the ad group criterion resource name for the parent listing group.
        # This can include a temporary ID if the parent criterion is not yet
        # created.
        listing_group_info.parent_ad_group_criterion = (
            parent_ad_group_criterion_resource_name
        )

        # Case values contain the listing dimension used for the node.
        client.copy_from(listing_group_info.case_value, listing_dimension_info)
    print("Subdivision Completed")
    return operation


def _create_listing_group_unit_biddable(
        client,
        customer_id,
        ad_group_id,
        parent_ad_group_criterion_resource_name,
        listing_dimension_info,
        cpc_bid_micros=None,
        included_product=False
):
    """Creates a new criterion containing a biddable unit listing group node.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID to which the node will be added.
        parent_ad_group_criterion_resource_name: The string resource name of the
            parent node to which this listing will be attached.
        listing_dimension_info: A ListingDimensionInfo object containing details
            for this listing.
        cpc_bid_micros: The cost-per-click bid for this listing in micros.

    Returns:
        An AdGroupCriterionOperation with a populated create field.
    """
    # Note: There are two approaches for creating new unit nodes:
    # (1) Set the ad group resource name on the criterion (no temporary ID
    # required).
    # (2) Use a temporary ID to construct the criterion resource name and set
    # it to the 'resourceName' attribute.
    # In both cases you must set the parent ad group criterion's resource name
    # on the listing group for non-root nodes.
    # This example demonstrates method (1).
    operation = client.get_type("AdGroupCriterionOperation")
    criterion = operation.create
    criterion.ad_group = client.get_service("AdGroupService").ad_group_path(
        customer_id, ad_group_id
    )

    criterion.status = client.get_type(
        "AdGroupCriterionStatusEnum"
    ).AdGroupCriterionStatus.ENABLED

    if included_product == True:
        criterion.negative = False
    elif included_product == False:
        criterion.negative = True
    else:
        print("There was a problem excluding your product tree")

    # Set the bid for this listing group unit.
    # This will be used as the CPC bid for items that are included in this
    # listing group.
    if cpc_bid_micros:
        criterion.cpc_bid_micros = cpc_bid_micros
    else:
        pass

    listing_group = criterion.listing_group
    # Set the type as a UNIT, which will allow the group to be biddable.
    listing_group.type_ = client.get_type(
        "ListingGroupTypeEnum"
    ).ListingGroupType.UNIT
    # Set the ad group criterion resource name for the parent listing group.
    # This can have a temporary ID if the parent criterion is not yet created.
    listing_group.parent_ad_group_criterion = (
        parent_ad_group_criterion_resource_name
    )
    # Case values contain the listing dimension used for the node.
    if listing_dimension_info != None:
        client.copy_from(listing_group.case_value, listing_dimension_info)

    return operation

def _get_ad_group_id(client, customer_id, ad_group_resource_name):

    googleservice = client.get_service("GoogleAdsService")
    query = f"""
              SELECT ad_group.id, ad_group.cpc_bid_micros
              FROM ad_group 
              WHERE ad_group.resource_name = '{ad_group_resource_name}'

      """

    #get ad_group_id and the cpc that was assigned to the ad group.
    search_request = client.get_type("SearchGoogleAdsStreamRequest")
    search_request.customer_id = customer_id
    search_request.query = query
    response = googleservice.search_stream(search_request)
    for batch in response:
        for row in batch.results:
            ad_group = row.ad_group
            ad_group_id = ad_group.id

    return ad_group_id

def _does_campaign_exist(client, customer_id, campaign_name):
    """Function which queries if campaign name already exists"""
    googleservice = client.get_service("GoogleAdsService")

    query = f"""
            SELECT campaign.name, campaign.id, campaign.status 
            FROM campaign
            WHERE campaign.name = '{campaign_name} Irrelevant'
    """
    #get campaign name.
    search_request = client.get_type("SearchGoogleAdsStreamRequest")
    search_request.customer_id = customer_id
    search_request.query = query
    response = googleservice.search_stream(search_request)
    for batch in response:
        for row in batch.results:
            campaign = row.campaign
            camp_name = campaign.name
            campaign_id = campaign.id
            campaign_status = campaign.status

        if camp_name == f"{campaign_name} Irrelevant":
            return True




if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    yaml_filepath = '//DDSERVER/Downloads$/googleadscode/googleads.yaml'
    # yaml_filepath = r"C:\Users\ASUS\Downloads\AutomationGoogleAds (1)\AutomationGoogleAds\theyamlfile\updatedgoogleads.yaml"
    customer_id = '5927843196'
    merchant_center_account_id = 107945304
    googleads_client = GoogleAdsClient.load_from_storage(yaml_filepath, version="v14")
    #query_ad_group_id(googleads_client, customer_id)
    ad_group_id = "12696561759"
    listing_array = ["10119", "10120"]
    replace_existing_tree = True
    print(_does_campaign_exist(googleads_client, customer_id, "test"))
    #
    # try:
    #     _page2(googleads_client, customer_id, merchant_center_account_id, listing_array)
    #
    # except GoogleAdsException as ex:
    #     print(
    #         f"Request with ID '{ex.request_id}' failed with status "
    #         f"'{ex.error.code().name}' and includes the following errors:"
    #     )
    #     for error in ex.failure.errors:
    #         print(f"\tError with message '{error.message}'.")
    #         if error.location:
    #             for field_path_element in error.location.field_path_elements:
    #                 print(f"\t\tOn field: {field_path_element.field_name}")
    #     sys.exit(1)
