import numpy as np
import xlrd
from openpyxl import load_workbook
import pandas as pd

def _importexcelfile(filepath):

    sheetpage = 'Sheet1'
    #Reads data from excel file into pandas dataframe.
    data = pd.read_excel(rf'{filepath}', sheet_name=sheetpage,
                         index_col=None, header=0, na_values=['NA'])
    #Get all values from first column, removing NA values
    irrelevant_column = data.iloc[:, 0]
    irrelevant_column = irrelevant_column.dropna()

    #Get all values from column 4 (all negatives) and remove na values
    allnegative_column = data.iloc[:,3]
    allnegative_column = allnegative_column.dropna()

    #Create df for broad column and drop na values.
    broad_column = data.iloc[:, 1]
    broad_column = broad_column.dropna()

    #Create df for broad column and drop na values.
    exact_column = data.iloc[:, 2]
    exact_column = exact_column.dropna()

    #merge irrelevant column and all negatives into one dataframe with one column.
    irrelevant_column_concat = [irrelevant_column, allnegative_column]
    irrelevant_column_concat = pd.concat(irrelevant_column_concat, ignore_index=True)
    irrelevantCamp_keywords = _merge_into_keyword_list(irrelevant_column_concat)

    #merge irrelevant column and all negatives into one dataframe with one column.
    broad_column_concat = [broad_column, allnegative_column]
    broad_column_concat = pd.concat(broad_column_concat, ignore_index=True)
    broadCamp_keywords = _merge_into_keyword_list(broad_column_concat)

    #merge irrelevant column and all negatives into one dataframe with one column.
    exact_column_concat = [exact_column, allnegative_column]
    exact_column_concat = pd.concat(exact_column_concat, ignore_index=True)
    exactCamp_keywords = _merge_into_keyword_list(exact_column_concat)

    return irrelevantCamp_keywords, broadCamp_keywords, exactCamp_keywords


def _merge_into_keyword_list(campaign_keywords):

    #The character to look for to identify a phrase keyword.
    phrase_splitword = "\""
    exact_splitword = "["

    #Finding the entries in broad column concat that start wih " and place them in new series, removing na values from deleted entries.
    prioritybroad_phrase = campaign_keywords.where(campaign_keywords.str.startswith(phrase_splitword)).dropna()
    prioritybroad_exact = campaign_keywords.where(campaign_keywords.str.startswith(exact_splitword)).dropna()
    #Creating the broad keyword list by finding the entries that are not in the above two series. (with other words, removing the exact and phrase keywords)
    prioritybroad_broad = campaign_keywords[~campaign_keywords.isin(prioritybroad_phrase)]
    prioritybroad_broad = prioritybroad_broad[~prioritybroad_broad.isin(prioritybroad_exact)].reset_index(drop=True)

    #Remove the quotations on both sides.
    prioritybroad_phrase = prioritybroad_phrase.str.replace('"', '')
    #Remove square brackets on both sides.
    prioritybroad_exact = prioritybroad_exact.str.strip('[]')

    #at this point you have prioritybroad_broad, prioritybroad_phrase, prioritybroad_exact containing all the keywords with different match types for the campaign.

    return prioritybroad_broad, prioritybroad_phrase, prioritybroad_exact

def _slice_dataframe_by_matchtype(keyword_dataframe, MatchType):

    mask = keyword_dataframe['MatchType'].values == MatchType
    #Retrieved entries are stored in keyword_dataframe_sorted
    keyword_dataframe_sorted = keyword_dataframe[mask]
    print(keyword_dataframe_sorted)
    return keyword_dataframe_sorted

if __name__ == '__main__':

    data = _importexcelfile(r'C:\Users\malin.sundstrom\Downloads\Parasolnegativekeywords.xlsx')
    data_irrelevantcamp = data[0]
    if isinstance(data[0], pd.Series):
        print("I am a series")

    data_broadcamp = data[1]
    data_exactcamp = data[2]
    print(data_irrelevantcamp)